@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2006,2008 by Solar Designer
+ * Copyright (c) 1996-2001,2006,2008,2010 by Solar Designer
  */
 
 #include <stdio.h>
@@ -66,8 +66,9 @@ char *fmt_self_test(struct fmt_main *format)
 
 		format->methods.crypt_all(index + 1);
 
-		for (size = 0; size < 3; size++)
-		if (format->methods.get_hash[size](index) !=
+		for (size = 0; size < PASSWORD_HASH_SIZES; size++)
+		if (format->methods.binary_hash[size] &&
+		    format->methods.get_hash[size](index) !=
 		    format->methods.binary_hash[size](binary)) {
 			sprintf(s_size, "get_hash[%d](%d)", size, index);
 			return s_size;
