@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2003,2005 by Solar Designer
+ * Copyright (c) 1996-2000,2003,2005,2010 by Solar Designer
  */
 
 #include <stdio.h>
@@ -89,9 +89,11 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 
 	db->salts = NULL;
 
+	db->password_hash = NULL;
+	db->password_hash_func = NULL;
+
 	if (options->flags & DB_CRACKED) {
 		db->salt_hash = NULL;
-		db->password_hash = NULL;
 
 		db->cracked_hash = mem_alloc(
 			CRACKED_HASH_SIZE * sizeof(struct db_cracked *));
@@ -103,9 +105,6 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 		memset(db->salt_hash, 0,
 			SALT_HASH_SIZE * sizeof(struct db_salt *));
 
-		db->password_hash = mem_alloc(LDR_HASH_SIZE);
-		memset(db->password_hash, 0, LDR_HASH_SIZE);
-
 		db->cracked_hash = NULL;
 
 		if (options->flags & DB_WORDS) {
@@ -115,9 +114,6 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 		}
 	}
 
-	if (!options->max_pps)
-		db->options->max_pps = (unsigned int)~0 >> 1;
-
 	list_init(&db->plaintexts);
 
 	db->salt_count = db->password_count = db->guess_count = 0;
@@ -125,6 +121,34 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 	db->format = NULL;
 }
 
+/*
+ * Allocate a hash table for use by the loader itself.  We use this for two
+ * purposes: to detect and avoid loading of duplicate hashes when DB_WORDS is
+ * not set, and to remove previously-cracked hashes (found in john.pot).  We
+ * allocate, use, and free this hash table prior to deciding on the sizes of
+ * and allocating the per-salt hash tables to be used while cracking.
+ */
+static void ldr_init_password_hash(struct db_main *db)
+{
+	int (*func)(void *binary);
+	int size = PASSWORD_HASH_SIZES - 1;
+
+	if (mem_saving_level >= 2)
+		size--;
+
+	do {
+		func = db->format->methods.binary_hash[size];
+		if (func && func != fmt_default_binary_hash)
+			break;
+	} while (--size >= 0);
+	if (size < 0)
+		size = 0;
+	db->password_hash_func = func;
+	size = password_hash_sizes[size] * sizeof(struct db_password *);
+	db->password_hash = mem_alloc(size);
+	memset(db->password_hash, 0, size);
+}
+
 static char *ldr_get_field(char **ptr)
 {
 	char *res, *pos;
@@ -209,7 +233,7 @@ static int ldr_split_line(char **login, char **ciphertext,
 		if (source) sprintf(source, "%s:%s", uid, line);
 	}
 
-	if (options->flags & DB_WORDS || options->shells->head) {
+	if ((options->flags & DB_WORDS) || options->shells->head) {
 		gid = ldr_get_field(&line);
 		do {
 			*gecos = ldr_get_field(&line);
@@ -242,6 +266,7 @@ static int ldr_split_line(char **login, char **ciphertext,
 static void ldr_split_string(struct list_main *dst, char *src)
 {
 	char *word, *pos;
+	char c;
 
 	pos = src;
 	do {
@@ -251,10 +276,11 @@ static void ldr_split_string(struct list_main *dst, char *src)
 
 		pos = word;
 		while (!issep_map[ARCH_INDEX(*pos)]) pos++;
-		if (*pos) *pos++ = 0;
-
+		c = *pos;
+		*pos = 0;
 		list_add_unique(dst, word);
-	} while (*pos && dst->count < LDR_WORDS_MAX);
+		*pos++ = c;
+	} while (c && dst->count < LDR_WORDS_MAX);
 }
 
 static struct list_main *ldr_init_words(char *login, char *gecos, char *home)
@@ -264,11 +290,12 @@ static struct list_main *ldr_init_words(char *login, char *gecos, char *home)
 
 	list_init(&words);
 
-	list_add(words, login);
+	if (*login)
+		list_add(words, login);
 	ldr_split_string(words, gecos);
 	ldr_split_string(words, login);
 
-	if ((pos = strrchr(home, '/')))
+	if ((pos = strrchr(home, '/')) && pos[1])
 		list_add_unique(words, pos + 1);
 
 	return words;
@@ -310,23 +337,26 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 			sizeof(struct db_keys *);
 	}
 
+	if (!db->password_hash)
+		ldr_init_password_hash(db);
+
 	for (index = 0; index < count; index++) {
 		piece = format->methods.split(ciphertext, index);
 
 		binary = format->methods.binary(piece);
-		pw_hash = LDR_HASH_FUNC(binary);
+		pw_hash = db->password_hash_func(binary);
 
-		if ((current_pw = db->password_hash[pw_hash]))
-		do {
-			if (!memcmp(current_pw->binary, binary,
-			    format->params.binary_size) &&
-			    !strcmp(current_pw->source, piece)) {
-				if (!(db->options->flags & DB_WORDS) ||
-				    !strcmp(current_pw->login, login)) break;
-			}
-		} while ((current_pw = current_pw->next_hash));
+		if (!(db->options->flags & DB_WORDS)) {
+			if ((current_pw = db->password_hash[pw_hash]))
+			do {
+				if (!memcmp(current_pw->binary, binary,
+				    format->params.binary_size) &&
+				    !strcmp(current_pw->source, piece))
+					break;
+			} while ((current_pw = current_pw->next_hash));
 
-		if (current_pw) continue;
+			if (current_pw) continue;
+		}
 
 		salt = format->methods.salt(piece);
 		salt_hash = format->methods.salt_hash(salt);
@@ -385,13 +415,13 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 		}
 
 		if (db->options->flags & DB_LOGIN) {
-			if (count > 1) {
+			if (count >= 2 && count <= 9) {
 				current_pw->login = mem_alloc_tiny(
 					strlen(login) + 3, MEM_ALIGN_NONE);
 				sprintf(current_pw->login, "%s:%d",
 					login, index + 1);
 			} else
-			if (words)
+			if (words && *login)
 				current_pw->login = words->head->data;
 			else
 				current_pw->login = str_alloc_copy(login);
@@ -417,7 +447,7 @@ static void ldr_load_pot_line(struct db_main *db, char *line)
 
 	ciphertext = format->methods.split(ciphertext, 0);
 	binary = format->methods.binary(ciphertext);
-	hash = LDR_HASH_FUNC(binary);
+	hash = db->password_hash_func(binary);
 
 	if ((current = db->password_hash[hash]))
 	do {
@@ -434,6 +464,15 @@ void ldr_load_pot_file(struct db_main *db, char *name)
 		read_file(db, name, RF_ALLOW_MISSING, ldr_load_pot_line);
 }
 
+/*
+ * The following are several functions called by ldr_fix_database().
+ * They assume that the per-salt hash tables have not yet been initialized.
+ */
+
+/*
+ * Glue the salt_hash[] buckets together and into the salts list.  The loader
+ * needs the hash table, but then we free it and the cracker uses the list.
+ */
 static void ldr_init_salts(struct db_main *db)
 {
 	struct db_salt **tail, *current;
@@ -448,6 +487,10 @@ static void ldr_init_salts(struct db_main *db)
 	}
 }
 
+/*
+ * Remove the previously-cracked hashes marked with "binary = NULL" by
+ * ldr_load_pot_line().
+ */
 static void ldr_remove_marked(struct db_main *db)
 {
 	struct db_salt *current_salt, *last_salt;
@@ -483,15 +526,24 @@ static void ldr_remove_marked(struct db_main *db)
 	} while ((current_salt = current_salt->next));
 }
 
+/*
+ * Remove salts with too few or too many password hashes.
+ */
 static void ldr_filter_salts(struct db_main *db)
 {
 	struct db_salt *current, *last;
+	int min = db->options->min_pps;
+	int max = db->options->max_pps;
+
+	if (!max) {
+		if (!min) return;
+		max = ~(unsigned int)0 >> 1;
+	}
 
 	last = NULL;
 	if ((current = db->salts))
 	do {
-		if (current->count < db->options->min_pps ||
-		    current->count > db->options->max_pps) {
+		if (current->count < min || current->count > max) {
 			if (last)
 				last->next = current->next;
 			else
@@ -504,7 +556,11 @@ static void ldr_filter_salts(struct db_main *db)
 	} while ((current = current->next));
 }
 
-void ldr_update_salt(struct db_main *db, struct db_salt *salt)
+/*
+ * Allocate memory for and initialize the hash table for this salt if needed.
+ * Also initialize salt->count (the number of password hashes for this salt).
+ */
+static void ldr_init_hash_for_salt(struct db_main *db, struct db_salt *salt)
 {
 	struct db_password *current;
 	int (*hash_func)(void *binary);
@@ -514,16 +570,19 @@ void ldr_update_salt(struct db_main *db, struct db_salt *salt)
 		salt->count = 0;
 		if ((current = salt->list))
 		do {
-			current->next_hash = current->next;
+			current->next_hash = NULL; /* unused */
 			salt->count++;
 		} while ((current = current->next));
 
 		return;
 	}
 
-	memset(salt->hash, 0,
-		password_hash_sizes[salt->hash_size] *
-		sizeof(struct db_password *));
+	salt->hash = mem_alloc_tiny(password_hash_sizes[salt->hash_size] *
+	    sizeof(struct db_password *), MEM_ALIGN_WORD);
+	memset(salt->hash, 0, password_hash_sizes[salt->hash_size] *
+	    sizeof(struct db_password *));
+
+	salt->index = db->format->methods.get_hash[salt->hash_size];
 
 	hash_func = db->format->methods.binary_hash[salt->hash_size];
 
@@ -537,30 +596,49 @@ void ldr_update_salt(struct db_main *db, struct db_salt *salt)
 	} while ((current = current->next));
 }
 
+/*
+ * Decide on whether to use a hash table and on its size for each salt, call
+ * ldr_init_hash_for_salt() to allocate and initialize the hash tables.
+ */
 static void ldr_init_hash(struct db_main *db)
 {
 	struct db_salt *current;
-	int size;
+	int threshold, size;
+
+	threshold = password_hash_thresholds[0];
+	if (db->format && (db->format->params.flags & FMT_BS)) {
+/*
+ * Estimate the complexity of DES_bs_get_hash() for each computed hash (but
+ * comparing it against less than 1 loaded hash on average due to the use of a
+ * hash table) vs. the complexity of DES_bs_cmp_all() for all computed hashes
+ * at once (but calling it for each loaded hash individually).
+ */
+		threshold = db->format->params.min_keys_per_crypt * 5;
+#if DES_BS_VECTOR
+		threshold /= ARCH_BITS_LOG * DES_BS_VECTOR;
+#else
+		threshold /= ARCH_BITS_LOG;
+#endif
+		threshold++;
+	}
 
 	if ((current = db->salts))
 	do {
-		for (size = 2; size >= 0; size--)
-		if (current->count >= password_hash_thresholds[size]) break;
+		size = -1;
+		if (current->count >= threshold && mem_saving_level < 3)
+			for (size = PASSWORD_HASH_SIZES - 1; size >= 0; size--)
+				if (current->count >=
+				    password_hash_thresholds[size] &&
+				    db->format->methods.binary_hash[size] &&
+				    db->format->methods.binary_hash[size] !=
+				    fmt_default_binary_hash)
+					break;
 
-		if ((db->format->params.flags & FMT_BS) && !size) size = -1;
+		if (mem_saving_level >= 2)
+			size--;
 
-		if (size >= 0 && mem_saving_level < 2) {
-			current->hash = mem_alloc_tiny(
-				password_hash_sizes[size] *
-				sizeof(struct db_password *),
-				MEM_ALIGN_WORD);
-
-			current->hash_size = size;
-
-			current->index = db->format->methods.get_hash[size];
-		}
-
-		ldr_update_salt(db, current);
+		current->hash_size = size;
+		ldr_init_hash_for_salt(db, current);
 	} while ((current = current->next));
 }
 
@@ -578,17 +656,78 @@ void ldr_fix_database(struct db_main *db)
 	db->loaded = 1;
 }
 
+/*
+ * ldr_remove_salt() is called by ldr_remove_hash() when it happens to remove
+ * the last password hash for a salt.
+ */
+static void ldr_remove_salt(struct db_main *db, struct db_salt *salt)
+{
+	struct db_salt **current;
+
+	db->salt_count--;
+
+	current = &db->salts;
+	while (*current != salt)
+		current = &(*current)->next;
+	*current = salt->next;
+}
+
+void ldr_remove_hash(struct db_main *db, struct db_salt *salt,
+	struct db_password *pw)
+{
+	struct db_password **current;
+	int hash;
+
+	db->password_count--;
+
+	if (!--salt->count) {
+		salt->list = NULL; /* "single crack" mode might care */
+		ldr_remove_salt(db, salt);
+		return;
+	}
+
+/*
+ * If there's no hash table for this salt, assume that next_hash fields are
+ * unused and don't need to be updated.  Only bother with the list.
+ */
+	if (salt->hash_size < 0) {
+		current = &salt->list;
+		while (*current != pw)
+			current = &(*current)->next;
+		*current = pw->next;
+		pw->binary = NULL;
+		return;
+	}
+
+	hash = db->format->methods.binary_hash[salt->hash_size](pw->binary);
+	current = &salt->hash[hash];
+	while (*current != pw)
+		current = &(*current)->next_hash;
+	*current = pw->next_hash;
+
+/*
+ * If there's a hash table for this salt, assume that the list is only used by
+ * "single crack" mode, so mark the entry for removal by "single crack" mode
+ * code in case that's what we're running, instead of traversing the list here.
+ */
+	pw->binary = NULL;
+}
+
 static int ldr_cracked_hash(char *ciphertext)
 {
-	int hash = 0;
+	unsigned int hash = 0;
+	char *p = ciphertext;
 
-	while (*ciphertext) {
+	while (*p) {
 		hash <<= 1;
-		hash ^= *ciphertext++ | 0x20; /* ASCII case insensitive */
+		hash += (unsigned char)*p++ | 0x20; /* ASCII case insensitive */
+		if (hash >> (2 * CRACKED_HASH_LOG - 1)) {
+			hash ^= hash >> CRACKED_HASH_LOG;
+			hash &= CRACKED_HASH_SIZE - 1;
+		}
 	}
 
 	hash ^= hash >> CRACKED_HASH_LOG;
-	hash ^= hash >> (2 * CRACKED_HASH_LOG);
 	hash &= CRACKED_HASH_SIZE - 1;
 
 	return hash;
@@ -603,6 +742,11 @@ static void ldr_show_pot_line(struct db_main *db, char *line)
 	ciphertext = ldr_get_field(&line);
 
 	if (line) {
+/* If just one format was forced on the command line, insist on it */
+		if (!fmt_list->next &&
+		    !fmt_list->methods.valid(ciphertext))
+			return;
+
 		pos = line;
 		do {
 			if (*pos == '\r' || *pos == '\n') *pos = 0;
@@ -649,6 +793,9 @@ static void ldr_show_pw_line(struct db_main *db, char *line)
 		source, &format, db->options, line);
 	if (!count) return;
 
+/* If just one format was forced on the command line, insist on it */
+	if (!fmt_list->next && !format) return;
+
 	show = !(db->options->flags & DB_PLAINTEXTS);
 
 	if (format) {
