@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99,2003,2005,2009 by Solar Designer
+ * Copyright (c) 1996-99,2003,2005,2009,2010 by Solar Designer
  */
 
 #include <stdio.h>
@@ -258,7 +258,7 @@ void rules_init(int max_length)
 	rules_init_length(max_length);
 }
 
-char *rules_reject(char *rule, int split, struct db_main *db)
+char *rules_reject(char *rule, int split, char *last, struct db_main *db)
 {
 	static char out_rule[RULE_BUFFER_SIZE];
 
@@ -309,7 +309,7 @@ char *rules_reject(char *rule, int split, struct db_main *db)
 accept:
 	rules_pass--;
 	strnzcpy(out_rule, rule - 1, sizeof(out_rule));
-	rules_apply("", out_rule, split, NULL);
+	rules_apply("", out_rule, split, last);
 	rules_pass++;
 
 	return out_rule;
@@ -905,15 +905,13 @@ out_OK:
 		if (length >= ARCH_SIZE - 1) {
 			if (*(ARCH_WORD *)in != *(ARCH_WORD *)last)
 				return in;
-			if (!strcmp(&in[ARCH_SIZE - 1], &last[ARCH_SIZE - 1]))
-				goto out_NULL;
-			return in;
+			if (strcmp(&in[ARCH_SIZE - 1], &last[ARCH_SIZE - 1]))
+				return in;
+			return NULL;
 		}
-		if (last[2])
+		if (last[length])
 			return in;
-		if (in[0] != last[0])
-			return in;
-		if (in[1] != last[1] && length)
+		if (memcmp(in, last, length))
 			return in;
 		return NULL;
 	}
@@ -967,7 +965,7 @@ int rules_check(struct rpp_context *start, int split)
 
 	rules_pass = -1; /* rules_reject() will turn this into -2 */
 	while ((rule = rpp_next(&ctx))) {
-		rules_reject(rule, split, NULL);
+		rules_reject(rule, split, NULL, NULL);
 		if (rules_errno) break;
 
 		if (ctx.input) rules_line = ctx.input->number;
